/*
 *   @(#) $Id: IoSessionConfig.java 378022 2006-02-15 15:30:18Z niklas $
 *
 *   Copyright 2004 The Apache Software Foundation
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 *
 */
package org.apache.mina.common;

import java.util.Set;

/**
 * The configuration of {@link IoSession}.
 * 
 * @author The Apache Directory Project (dev@directory.apache.org)
 * @version $Rev: 378022 $, $Date: 2006-02-16 00:30:18 +0900 (Thu, 16 Feb 2006) $
 */
public interface IoSessionConfig
{
    /**
     * Returns a deep clone of this configuration.
     */
    Object clone();
    
    /**
     * Returns the value of user-defined attribute of this session.
     * 
     * @param key the key of the attribute
     * @return <tt>null</tt> if there is no attribute with the specified key
     */
    Object getAttribute( String key );
    
    /**
     * Sets a user-defined attribute.
     * 
     * @param key the key of the attribute
     * @param value the value of the attribute
     * @return The old value of the attribute.  <tt>null</tt> if it is new.
     */
    Object setAttribute( String key, Object value );
    
    /**
     * Sets a user defined attribute without a value.  This is useful when
     * you just want to put a 'mark' attribute.  Its value is set to
     * {@link Boolean#TRUE}.
     * 
     * @param key the key of the attribute
     * @return The old value of the attribute.  <tt>null</tt> if it is new.
     */
    Object setAttribute( String key );
    
    /**
     * Removes a user-defined attribute with the specified key.
     * 
     * @return The old value of the attribute.  <tt>null</tt> if not found.
     */
    Object removeAttribute( String key );
    
    /**
     * Returns <tt>true</tt> if this session contains the attribute with
     * the specified <tt>key</tt>.
     */
    boolean containsAttribute( String key );
    
    /**
     * Returns the set of keys of all user-defined attributes.
     */
    Set getAttributeKeys();    
}
